//
//  ViewController.m
//  Tutorial
//
//  Created by Randy Palermo on 1/11/16.
//  Copyright (c) 2016 IDTech Products. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

@end

@implementation ViewController

@synthesize lcdTextView;
@synthesize logTextView;
@synthesize connectedLabel;

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    [[IDT_UniPayI_V sharedController] setDelegate:self];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void) deviceConnected{
    [connectedLabel setText:@"UniPayI_V CONNECTED"];
}
-(void) deviceDisconnected{
    [connectedLabel setText:@"UniPayI_V DISCONNECTED"];
}

-(void) appendMessageToResults:(NSString*) message{
    [self performSelectorOnMainThread:@selector(_appendMessageToResults:) withObject:message waitUntilDone:false];
    
}
-(void) _appendMessageToResults:(id)object{
    [logTextView setText:[NSString stringWithFormat:@"%@\n====================\n%@",(NSString*)object, self.logTextView.text]];
    
    // [self.resultsTextView scrollRangeToVisible:NSMakeRange([self.resultsTextView.text length], 0)];
    
}

- (void) swipeMSRData:(IDTMSRData*)cardData{
    NSLog(@"--MSR event Received, Type: %d, data: %@", cardData.event, cardData.encTrack1);
    switch (cardData.event) {
        case EVENT_MSR_CARD_DATA:
        {
            switch (cardData.captureEncodeType) {
                case CAPTURE_ENCODE_TYPE_ISOABA:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"ISO/ABA"]];
                    break;
                case CAPTURE_ENCODE_TYPE_AAMVA:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"AA/MVA"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Other:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"Other"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Raw:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"Raw"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_JIS_I:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"CAPTURE_ENCODE_TYPE_JIS_I"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_JIS_II:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"CAPTURE_ENCODE_TYPE_JIS_II"]];
                    break;
                    
                default:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encode Type: %@", @"UNKNOWN"]];
                    
                    break;
            }
            switch (cardData.captureEncryptType) {
                case CAPTURE_ENCRYPT_TYPE_AES:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encrypt Type: %@", @"AES"]];
                    break;
                case CAPTURE_ENCRYPT_TYPE_TDES:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encrypt Type: %@", @"TDES"]];
                    break;
                case CAPTURE_ENCRYPT_TYPE_NO_ENCRYPTION:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encrypt Type: %@", @"NONE"]];
                    break;
                    
                    
                default:
                    [self appendMessageToResults:[NSString stringWithFormat:@"Encrypt Type: %@", @"UNKNOWN"]];
                    
                    break;
            }
            
            [self appendMessageToResults:[NSString stringWithFormat:@"Full card data: %@", cardData.cardData]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Track 1: %@", cardData.track1]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Track 2: %@", cardData.track2]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Track 3: %@", cardData.track3]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Length Track 1: %i", cardData.track1Length]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Length Track 2: %i", cardData.track2Length]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Length Track 3: %i", cardData.track3Length]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Encoded Track 1: %@", cardData.encTrack1.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Encoded Track 2: %@", cardData.encTrack2.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Encoded Track 3: %@", cardData.encTrack3.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Hash Track 1: %@", cardData.hashTrack1.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Hash Track 2: %@", cardData.hashTrack2.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"Hash Track 3: %@", cardData.hashTrack3.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"KSN: %@", cardData.KSN.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"\nSessionID: %@",  cardData.sessionID.description]];
            [self appendMessageToResults:[NSString stringWithFormat:@"\nReader Serial Number: %@",  cardData.RSN]];
            [self appendMessageToResults:[NSString stringWithFormat:@"\nRead Status: %2X",  cardData.readStatus]];
            if (cardData.unencryptedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Unencrytped Tags: %@", cardData.unencryptedTags.description]];
            if (cardData.encryptedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Encrypted Tags: %@", cardData.encryptedTags.description]];
            if (cardData.maskedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Masked Tags: %@", cardData.maskedTags.description]];
            
            NSLog(@"Track 1: %@", cardData.track1);
            NSLog(@"Track 2: %@", cardData.track2);
            NSLog(@"Track 3: %@", cardData.track3);
            NSLog(@"Encoded Track 1: %@", cardData.encTrack1.description);
            NSLog(@"Encoded Track 2: %@", cardData.encTrack2.description);
            NSLog(@"Encoded Track 3: %@", cardData.encTrack3.description);
            NSLog(@"Hash Track 1: %@", cardData.hashTrack1.description);
            NSLog(@"Hash Track 2: %@", cardData.hashTrack2.description);
            NSLog(@"Hash Track 3: %@", cardData.hashTrack3.description);
            NSLog(@"SessionID: %@", cardData.sessionID.description);
            NSLog(@"nReader Serial Number: %@", cardData.RSN);
            NSLog(@"Read Status: %2X", cardData.readStatus);
            NSLog(@"KSN: %@", cardData.KSN.description);
            
            return;
        }
            break;
            
        case EVENT_MSR_CANCEL_KEY:
        {
            [self appendMessageToResults:[NSString stringWithFormat:@"(Event) MSR Cancel Key received: %@", cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_BACKSPACE_KEY:
        {
            [self appendMessageToResults:[NSString stringWithFormat:@"(Event) MSR Backspace Key received: %@", cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_ENTER_KEY:
        {
            [self appendMessageToResults:[NSString stringWithFormat:@"(Event) MSR Enter Key received: %@", cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_UNKNOWN:
        {
            [self appendMessageToResults:[NSString stringWithFormat:@"(Event) MSR unknown event, data: %@", cardData.encTrack1]];
            return;
        }
            break;
        case EVENT_MSR_TIMEOUT:
        {
            [self appendMessageToResults:@"(Event) MSR TIMEOUT"];
            return;
        }
        default:
            break;
    }
}

- (void) plugStatusChange:(BOOL)deviceInserted{
    if (deviceInserted) {
        [self appendMessageToResults: @"device Attached. Attempting connection"];
        [[IDT_UniPayI_V sharedController] device_connectToAudioReader];
       
    }
    else{
        [self appendMessageToResults: @"device removed."];
    }
}

- (void) emvTransactionData:(IDTEMVData*)emvData errorCode:(int)error{
    NSLog(@"EMV_RESULT_CODE_V2_response = %2X",error);
    
    [self appendMessageToResults:[NSString stringWithFormat:@"EMV_RESULT_CODE_V2_response = %2X",error]];
    if (emvData == nil) {
        [self appendMessageToResults:[NSString stringWithFormat:@"EMV TRANSACTION ERROR. Refer to EMV_RESULT_CODE_V2_response = 0x%2X",error]];
        return;
    }
    
    if (emvData.resultCodeV2 == EMV_RESULT_CODE_V2_GO_ONLINE) {
        [self appendMessageToResults:@"ONLINE REQUEST"];
    }
    if (emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED || emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED_OFFLINE ) {
        [self appendMessageToResults:@"APPROVED"];
    }
    if (emvData.resultCodeV2 == EMV_RESULT_CODE_V2_MSR_SUCCESS) {
        [self appendMessageToResults:@"MSR Data Captured"];
    }
    
    if (emvData.cardType == 0) {
        [self appendMessageToResults:@"CONTACT"];
    }
    if (emvData.cardType == 1) {
        [self appendMessageToResults:@"CONTACTLESS"];
    }
    
    if (emvData.unencryptedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Unencrytped Tags: %@", emvData.unencryptedTags.description]];
    if (emvData.encryptedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Encrypted Tags: %@", emvData.encryptedTags.description]];
    if (emvData.maskedTags != nil) [self appendMessageToResults:[NSString stringWithFormat:@"Masked Tags: %@", emvData.maskedTags.description]];
    if (emvData.cardData != nil) [self swipeMSRData:emvData.cardData];
}


- (void) lcdDisplay:(int)mode  lines:(NSArray*)lines{
    NSMutableString* str = [NSMutableString new];
    if (lines != nil) {
        for (NSString* s in lines) {
            [str appendString:s];
            [str appendString:@"\n"];
        }
    }
    
    switch (mode) {
        case 0x10:
            //clear screen
            lcdTextView.text = @"";
            break;
        case 0x03:
            lcdTextView.text = str;
            break;
        case 0x01:
        case 0x02:
        case 0x08:{
            [[IDT_UniPayI_V sharedController] emv_callbackResponseLCD:mode selection:(unsigned char)1];;
        }
            
            break;
        default:
            break;
    }
}


-(IBAction) getFirmware:(id)sender{
    NSString *result;
    logTextView.text = @"";
    RETURN_CODE rt = [[IDT_UniPayI_V sharedController]  device_getFirmwareVersion:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [self appendMessageToResults: [NSString stringWithFormat:@"Get FM info:  %@", result]];
    }
}
-(IBAction) startMSR_CTLS:(id)sender{
    logTextView.text = @"";
    RETURN_CODE rt = [[IDT_UniPayI_V sharedController] msr_startMSRSwipe];
    if(RETURN_CODE_DO_SUCCESS == rt){
        [self appendMessageToResults:@"EnableMSR: OK."];
    }
}
-(IBAction) startEMV:(id)sender{
    logTextView.text = @"";
    RETURN_CODE rt = [[IDT_UniPayI_V sharedController] emv_startTransaction:1.00 amtOther:0 type:0 timeout:60 tags:nil forceOnline:false  fallback:true];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [self appendMessageToResults: @"Start Transaction Command Accepted"];
    }
}
-(IBAction) completeEMV:(id)sender{
    logTextView.text = @"";
    RETURN_CODE rt = [[IDT_UniPayI_V sharedController]  emv_completeOnlineEMVTransaction:true hostResponseTags:[IDTUtility hexToData:@"8A023030"] ];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [self appendMessageToResults: @"Complete Transaction Command Accepted"];
    }
}
-(IBAction) cancelTransaction:(id)sender{
    logTextView.text = @"";
    RETURN_CODE rt = [[IDT_UniPayI_V sharedController] msr_cancelMSRSwipe];
    if(RETURN_CODE_DO_SUCCESS == rt){
        [self appendMessageToResults:@"CancelMSR: OK."];
    }
}

@end
