//
//  SecondViewController.swift
//  UniPayI_V_Test_App
//
//  Created by IDTech on 7/14/16.
//  Copyright © 2016 IDTech. All rights reserved.
//

import UIKit

class SecondViewController: UIViewController, IDT_UniPayI_V_Delegate, UITextFieldDelegate {
    
    var connection = Connection();

    @IBOutlet weak var connectionStatus: UILabel!
    @IBOutlet weak var resultsTextView: UITextView!
    @IBOutlet weak var dataTextView: UITextView!
    @IBOutlet weak var APDUInput: UITextField!
    @IBOutlet weak var commandInput: UITextField!
    @IBOutlet weak var subCommandInput: UITextField!
    @IBOutlet weak var dataInput: UITextField!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        self.APDUInput.delegate = self
        self.commandInput.delegate = self
        self.subCommandInput.delegate = self
        self.dataInput.delegate = self
        
        let tbvc = tabBarController as! TabBarController
        connection = tbvc.connection
        
        NotificationCenter.default.addObserver(self, selector: #selector(SecondViewController.reload), name: NSNotification.Name(rawValue: "load"), object: nil)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        connectionStatus.text = connection.getTheStatus()
        connectionStatus.backgroundColor = connection.getTheBackgroundColor()
        resultsTextView.text = connection.getTheResultsText()
        dataTextView.text = connection.getTheDataText()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        
        return false
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    func reload() {
        connectionStatus.text = connection.getTheStatus()
        connectionStatus.backgroundColor = connection.getTheBackgroundColor()
        resultsTextView.text = connection.getTheResultsText()
        dataTextView.text = connection.getTheDataText()
    }
    
    @IBAction func powerOnICC(_ sender: UIButton) {
        var response: NSData?
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().icc_power(onICC: &response)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("ICC powered on, data: \(response!.description)\n\(response!))", field: resultsTextView)
        } else {
            connection.displayReturnError("ICC power on", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func powerOffICC(_ sender: UIButton) {
        var error: NSString?
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().icc_powerOffICC(&error)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("ICC powered off", field: resultsTextView)
        } else {
            connection.displayReturnError("ICC power off", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func passThroughOn(_ sender: UIButton) {
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().device_setPassThrough(true)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Pass through on", field: resultsTextView)
        } else {
            connection.displayReturnError("Pass through on", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func passThroughOff(_ sender: UIButton) {
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().device_setPassThrough(false)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Pass through off", field: resultsTextView)
        } else {
            connection.displayReturnError("Pass through off", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func sendAPDU(_ sender: UIButton) {
        APDUInput.resignFirstResponder()
        
        var dataBuffer: Data? = nil;
        let valueEntered: NSString = APDUInput.text! as NSString
        
        if valueEntered.length < 6 {
            let alert: UIAlertView = UIAlertView(title: "Invalid", message: "Input APDU data in invalid", delegate: nil, cancelButtonTitle: "Dismiss")
            
            alert.show()
            return
        } else {
            dataBuffer = IDTUtility.hex(toData: valueEntered as String)
            
            if dataBuffer == nil || dataBuffer!.count < 3 {
                let alert: UIAlertView = UIAlertView(title: "Invalid", message: "Input APDU value is invalid", delegate: nil, cancelButtonTitle: "Dismiss")
                
                alert.show()
                return
            }
        }
        
        var response: APDUResponse?
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().icc_exchangeAPDU(dataBuffer, response: &response)
        
        connection.displayReturnError("Exchange APDU", rt: rt, field: resultsTextView)
        
        if (response != nil) {
            connection.appendMessageToResultsField("Exchange APDU result: APDU \(response!.response.description)", field: resultsTextView)
        }
    }
    
    @IBAction func sendCommand(_ sender: UIButton) {
        dataInput.resignFirstResponder()
        
        var commandValue: UInt32 = 0
        var scanner = Scanner.localizedScanner(with: commandInput.text!)
        (scanner as AnyObject).scanHexInt32(&commandValue)
        
        var subCommandValue: UInt32 = 0
        scanner = Scanner.localizedScanner(with: subCommandInput.text!)
        (scanner as AnyObject).scanHexInt32(&subCommandValue)
        
        let data = IDTUtility.hex(toData: dataInput.text)
        var response: NSData?
        let rt: RETURN_CODE = IDT_UniPayI_V.sharedController().device_sendIDGCommand(UInt8(commandValue), subCommand: UInt8(subCommandValue), data: data, response: &response)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Send command result: \(response!.description)", field: resultsTextView)
            connection.appendMessageToResultsField("Send command result ASCII: \(IDTUtility.data(toPrintableString: response as Data!))", field: resultsTextView)
        } else {
            connection.displayReturnError("Send command", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func clearOutputs(_ sender: UIBarButtonItem) {
        connection.clearOutputs(resultsTextView, dataTextView: dataTextView)
    }
}

